<?php

namespace Drupal\config_pages_overrides\Config;

use Drupal\Component\Utility\NestedArray;
use Drupal\config_pages\ConfigPagesLoaderServiceInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryOverrideInterface;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Configuration overrides for the profile.
 *
 * @package Drupal\stanford_profile_config_overrides\Config
 */
class ConfigOverrides implements ConfigFactoryOverrideInterface {

  /**
   * @var \Drupal\config_pages\ConfigPagesLoaderServiceInterface
   */
  protected $configPagesLoader;

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\config_pages\ConfigPagesTypeInterface[]
   */
  protected $configPagesTypes;

  /**
   * ConfigOverrides constructor.
   *
   * @param \Drupal\config_pages\ConfigPagesLoaderServiceInterface $config_pages
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(ConfigPagesLoaderServiceInterface $config_pages, EntityTypeManagerInterface $entity_type_manager) {
    $this->configPagesLoader = $config_pages;
    $this->configPagesTypes = $entity_type_manager->getStorage('config_pages_type')
      ->loadMultiple();
  }

  /**
   * {@inheritDoc}
   */
  public function loadOverrides($names) {
    $overrides = [];
    foreach ($this->configPagesTypes as $page_type) {
      foreach ($page_type->getThirdPartySettings('config_pages_overrides') as $settings) {
        if (in_array($settings['config_name'], $names)) {
          $path = [$settings['config_name']];
          $path = array_merge($path, explode('.', $settings['config_item']));

          $override_value = $this->configPagesLoader->getValue($page_type->id(), $settings['field'], $settings['delta'], $settings['column']);
          NestedArray::setValue($overrides, $path, $override_value);
        }
      }
    }
    return $overrides;
  }

  /**
   * {@inheritDoc}
   */
  public function createConfigObject($name, $collection = StorageInterface::DEFAULT_COLLECTION) {
    return NULL;
  }

  /**
   * {@inheritDoc}
   */
  public function getCacheableMetadata($name) {
    return new CacheableMetadata();
  }

  /**
   * {@inheritDoc}
   */
  public function getCacheSuffix() {
    return 'ConfigPagesOverrides';
  }

}
